// ignore_for_file: non_constant_identifier_names, camel_case_extensions, camel_case_extensions empty_catches, empty_catches

import 'dart:async';
import 'package:galaxeus_lib/galaxeus_lib.dart';
import 'package:telegram_client/telegram_client/function/answer_callback_query.dart';
import 'package:telegram_client/telegram_client/function/delete_message.dart';
import 'package:telegram_client/telegram_client/function/edit_message_caption.dart';
import 'package:telegram_client/telegram_client/function/edit_message_media.dart';
import 'package:telegram_client/telegram_client/function/edit_message_reply_markup.dart';
import 'package:telegram_client/telegram_client/function/edit_message_text.dart';
import 'package:telegram_client/telegram_client/function/function.dart';
import 'package:telegram_client/tdlib/tdlib_ffi/tdlib.dart';
import 'package:telegram_client/telegram_bot_api/update_bot.dart';
import 'package:telegram_client/telegram_client.dart';
import 'package:telegram_client/telegram_client/function/get_chat_administrators.dart';
import 'package:telegram_client/telegram_client/function/get_chat_member.dart';
import 'package:telegram_client/telegram_client/function/get_me.dart';
import 'package:telegram_client/telegram_client/function/send_animation.dart';
import 'package:telegram_client/telegram_client/function/send_audio.dart';
import 'package:telegram_client/telegram_client/function/send_chat_action.dart';
import 'package:telegram_client/telegram_client/function/send_contact.dart';
import 'package:telegram_client/telegram_client/function/send_dice.dart';
import 'package:telegram_client/telegram_client/function/send_document.dart';
import 'package:telegram_client/telegram_client/function/send_game.dart';
import 'package:telegram_client/telegram_client/function/send_invoice.dart';
import 'package:telegram_client/telegram_client/function/send_location.dart';
import 'package:telegram_client/telegram_client/function/send_media_group.dart';
import 'package:telegram_client/telegram_client/function/send_message.dart';
import 'package:telegram_client/telegram_client/function/send_photo.dart';
import 'package:telegram_client/telegram_client/function/send_poll.dart';
import 'package:telegram_client/telegram_client/function/send_sticker.dart';
import 'package:telegram_client/telegram_client/function/send_venue.dart';
import 'package:telegram_client/telegram_client/function/send_video.dart';
import 'package:telegram_client/telegram_client/function/send_video_note.dart';
import 'package:telegram_client/telegram_client/function/send_voice.dart';
import "package:telegram_client/telegram_client/scheme/scheme.dart"
    as telegram_client_scheme;
import 'package:universal_io/io.dart';

/// return original data json
class TelegramClient {
  late Tdlib tdlib;
  late TelegramBotApi telegramBotApi;
  EventEmitter event_emitter = EventEmitter();
  String event_update;
  String event_invoke;
  Directory? directory_temp;

  /// return original data json
  TelegramClient({
    EventEmitter? eventEmitter,
    this.event_invoke = "tg_client_invoke",
    this.event_update = "tg_client_update",
    this.directory_temp,
  }) {
    if (eventEmitter != null) {
      event_emitter = eventEmitter;
    }
  }

  /// return original data json
  Future<void> init({
    String? pathTdlib,
    TelegramClientTdlibOption? telegramClientTdlibOption,
    TelegramClientTelegramBotApiOption? telegramClientTelegramBotApiOption,
  }) async {
    telegramClientTdlibOption ??= TelegramClientTdlibOption(
      isAutoGetChat: false,
      is_cli: false,
      timeOutUpdate: 1.0,
      delayInvoke: Duration(milliseconds: 1),
      delayUpdate: Duration(milliseconds: 1),
      isInvokeThrowOnError: false,
    );
    telegramClientTelegramBotApiOption ??= TelegramClientTelegramBotApiOption(
      clientOption: {},
    );
    telegramBotApi = TelegramBotApi(
      tokenBot: "",
      event_invoke: event_invoke,
      event_update: event_update,
      clientOption: telegramClientTelegramBotApiOption.clientOption,
      eventEmitter: event_emitter,
      alfred: telegramClientTelegramBotApiOption.alfred,
      telegramCryptoKey: telegramClientTelegramBotApiOption.telegramCryptoKey,
      telegramUrlWebhook: telegramClientTelegramBotApiOption.telegramUrlWebhook,
    );
    tdlib = Tdlib(
      pathTdl: pathTdlib,
      is_cli: telegramClientTdlibOption.is_cli,
      clientOption: telegramClientTdlibOption.clientOption,
      clientId: telegramClientTdlibOption.clientId,
      invokeTimeOut: telegramClientTdlibOption.invokeTimeOut,
      event_invoke: event_invoke,
      event_update: event_update,
      delayUpdate: telegramClientTdlibOption.delayUpdate,
      delayInvoke: telegramClientTdlibOption.delayInvoke,
      isAutoGetChat: telegramClientTdlibOption.isAutoGetChat,
      on_generate_extra_invoke:
          telegramClientTdlibOption.on_generate_extra_invoke,
      on_get_invoke_data: telegramClientTdlibOption.on_get_invoke_data,
      on_receive_update: telegramClientTdlibOption.on_receive_update,
      isInvokeThrowOnError: telegramClientTdlibOption.isInvokeThrowOnError,
      eventEmitter: event_emitter,
    );
  }

  /// return original data json
  Listener on({
    required String event_name,
    required FutureOr<dynamic> Function(
            UpdateTelegramClient updateTelegramClient)
        onUpdate,
    required FutureOr<dynamic> Function(Object error, StackTrace stackTrace)
        onError,
  }) {
    return event_emitter.on(event_name, null, (ev, context) async {
      try {
        if (ev.eventData is UpdateBot) {
          UpdateBot updateBot = (ev.eventData as UpdateBot);
          await onUpdate(
            UpdateTelegramClient(
              rawData: updateBot.body,
              query: updateBot.query,
              client_option: {},
              tg: this,
              telegramClientData: TelegramClientData(
                tdlib_client_id: 0,
                telegram_bot_api_token_bot: "",
                telegramClientType: TelegramClientType.telegam_bot_api,
              ),
            ),
          );
        }

        if (ev.eventData is TdlibIsolateReceiveData) {
          TdlibIsolateReceiveData tdlibIsolateReceiveData =
              (ev.eventData as TdlibIsolateReceiveData);

          TdlibClient? tdlibClient =
              tdlib.clients[tdlibIsolateReceiveData.clientId];

          UpdateTd updateTd = UpdateTd(
            update: tdlibIsolateReceiveData.updateData,
            client_id: tdlibIsolateReceiveData.clientId,
            client_option: () {
              try {
                if (tdlibClient != null) {
                  return tdlibClient.client_option;
                }
              } catch (e) {}
              return {};
            }(),
          );
          await onUpdate(
            UpdateTelegramClient(
              rawData: updateTd.raw,
              query: {},
              tg: this,
              client_option: updateTd.client_option,
              telegramClientData: TelegramClientData(
                tdlib_client_id: updateTd.client_id,
                telegram_bot_api_token_bot: "",
                telegramClientType: TelegramClientType.tdlib,
                is_bot: () {
                  try {
                    if (tdlibClient != null) {
                      return tdlibClient.is_bot;
                    }
                  } catch (e) {}
                  return false;
                }(),
              ),
            ),
          );
        }
      } catch (e, stack) {
        try {
          await onError(e, stack);
        } catch (e) {}
      }
    });
  }

  /// return original data json
  FutureOr<telegram_client_scheme.JsonScheme> invoke({
    required Map parameters,
    required TelegramClientData telegramClientData,
    bool is_form = false,
    String? urlApi,
    String? clientType,
    void Function(int bytes, int bytess)? onUploadProgress,
    bool isVoid = false,
    Duration? delayDuration,
    Duration? invokeTimeOut,
    String? extra,
    bool? isAutoGetChat,
    bool isInvokeThrowOnError = true,
    FutureOr<String> Function(int client_id, LibTdJson libTdJson)?
        onGenerateExtraInvoke,
    FutureOr<Map<dynamic, dynamic>> Function(
            String, int client_id, LibTdJson libTdJson)?
        onGetInvokeData,
  }) async {
    // telegramClientLib ??= telegram_client_lib;
    if (telegramClientData.telegramClientType ==
        TelegramClientType.telegam_bot_api) {
      Map respond = await telegramBotApi.invoke(
        parameters["@type"],
        tokenBot: telegramClientData.telegram_bot_api_token_bot,
        parameters: parameters,
        is_form: is_form,
        urlApi: urlApi,
        clientType: clientType,
        isThrowOnError: isInvokeThrowOnError,
        onUploadProgress: onUploadProgress,
      );
      if (respond["@type"] == "error") {
        return telegram_client_scheme.Error(respond);
      }
      return telegram_client_scheme.JsonScheme(respond);
    }
    if ((telegramClientData.telegramClientType == TelegramClientType.tdlib)) {
      Map respond = (await tdlib.invoke(
        parameters["@type"],
        parameters: parameters,
        clientId: telegramClientData.tdlib_client_id,
        isVoid: isVoid,
        delayDuration: delayDuration,
        invokeTimeOut: invokeTimeOut,
        extra: extra,
        isAutoGetChat: isAutoGetChat,
        isInvokeThrowOnError: isInvokeThrowOnError,
        onGenerateExtraInvoke: onGenerateExtraInvoke,
        onGetInvokeData: onGetInvokeData,
      ));
      if (respond["@type"] == "error") {
        return telegram_client_scheme.Error(respond);
      }
      return telegram_client_scheme.JsonScheme(respond);
    }
    telegram_client_scheme.Error error = telegram_client_scheme.Error.create(
      special_extra: "error",
      code: 500,
      message: "telegram_client_type_not_found",
      description: "",
    );
    if (isInvokeThrowOnError == true) {
      throw error;
    }
    return error;
  }

  /// return original data json
  FutureOr<telegram_client_scheme.JsonScheme> request({
    required Map parameters,
    required TelegramClientData telegramClientData,
    bool is_form = false,
    String? urlApi,
    String? clientType,
    void Function(int bytes, int bytess)? onUploadProgress,
    bool isVoid = false,
    Duration? delayDuration,
    Duration? invokeTimeOut,
    String? extra,
    bool? isAutoGetChat,
    bool isInvokeThrowOnError = true,
    FutureOr<String> Function(int client_id, LibTdJson libTdJson)?
        onGenerateExtraInvoke,
    FutureOr<Map<dynamic, dynamic>> Function(
            String, int client_id, LibTdJson libTdJson)?
        onGetInvokeData,
  }) async {
    if (telegramClientData.telegramClientType != TelegramClientType.tdlib) {
      return await invoke(
        parameters: parameters,
        telegramClientData: telegramClientData,
        is_form: is_form,
        urlApi: urlApi,
        clientType: clientType,
        onUploadProgress: onUploadProgress,
        isVoid: isVoid,
        delayDuration: delayDuration,
        invokeTimeOut: invokeTimeOut,
        extra: extra,
        isAutoGetChat: isAutoGetChat,
        isInvokeThrowOnError: isInvokeThrowOnError,
        onGenerateExtraInvoke: onGenerateExtraInvoke,
        onGetInvokeData: onGetInvokeData,
      );
    }
    String method = (parameters["@type"] as String);

    /// return original data json
    FutureOr<telegram_client_scheme.JsonScheme> callApiInvoke({
      required Map parameters,
    }) async {
      return await invoke(
        parameters: parameters,
        telegramClientData: telegramClientData,
        is_form: is_form,
        urlApi: urlApi,
        clientType: clientType,
        onUploadProgress: onUploadProgress,
        isVoid: isVoid,
        delayDuration: delayDuration,
        invokeTimeOut: invokeTimeOut,
        extra: extra,
        isAutoGetChat: isAutoGetChat,
        isInvokeThrowOnError: isInvokeThrowOnError,
        onGenerateExtraInvoke: onGenerateExtraInvoke,
        onGetInvokeData: onGetInvokeData,
      );
    }

    if (RegExp(r"^(answerCallbackQuery)$", caseSensitive: false)
        .hasMatch(method)) {
      return await answerCallbackQuery(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
      );
    }
    if (RegExp(r"^(deleteMessage)$", caseSensitive: false).hasMatch(method)) {
      return await deleteMessage(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
      );
    }
    if (RegExp(r"^(editMessageText)$", caseSensitive: false).hasMatch(method)) {
      return await editMessageText(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }
    if (RegExp(r"^(editMessageCaption)$", caseSensitive: false)
        .hasMatch(method)) {
      return await editMessageCaption(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }
    if (RegExp(r"^(editMessageMedia)$", caseSensitive: false)
        .hasMatch(method)) {
      return await editMessageMedia(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }
    if (RegExp(r"^(editMessageReplyMarkup)$", caseSensitive: false)
        .hasMatch(method)) {
      return await editMessageReplyMarkup(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }
    if (RegExp(r"^(getChatAdministrators)$", caseSensitive: false)
        .hasMatch(method)) {
      return await getChatAdministrators(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
      );
    }
    if (RegExp(r"^(getChatMember)$", caseSensitive: false).hasMatch(method)) {
      return await getChatMember(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
      );
    }
    if (RegExp(r"^(getChat)$", caseSensitive: false).hasMatch(method)) {
      return await getChat(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
      );
    }
    if (RegExp(r"^(getMe)$", caseSensitive: false).hasMatch(method)) {
      return await getMe(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
      );
    }
    if (RegExp(r"^(getUser)$", caseSensitive: false).hasMatch(method)) {
      return await getUser(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
      );
    }

    if (RegExp(r"^(sendAnimation)$", caseSensitive: false).hasMatch(method)) {
      return await sendAnimation(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }
    if (RegExp(r"^(sendAudio)$", caseSensitive: false).hasMatch(method)) {
      return await sendAudio(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }
    if (RegExp(r"^(sendChatAction)$", caseSensitive: false).hasMatch(method)) {
      return await sendChatAction(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }
    if (RegExp(r"^(sendContact)$", caseSensitive: false).hasMatch(method)) {
      return await sendContact(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }
    if (RegExp(r"^(sendDice)$", caseSensitive: false).hasMatch(method)) {
      return await sendDice(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }
    if (RegExp(r"^(sendDocument)$", caseSensitive: false).hasMatch(method)) {
      return await sendDocument(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }
    if (RegExp(r"^(sendGame)$", caseSensitive: false).hasMatch(method)) {
      return await sendGame(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }

    if (RegExp(r"^(sendInvoice)$", caseSensitive: false).hasMatch(method)) {
      return await sendInvoice(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }

    if (RegExp(r"^(sendLocation)$", caseSensitive: false).hasMatch(method)) {
      return await sendLocation(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }

    if (RegExp(r"^(sendMediaGroup)$", caseSensitive: false).hasMatch(method)) {
      return await sendMediaGroup(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }

    if (RegExp(r"^(sendMessage)$", caseSensitive: false).hasMatch(method)) {
      return await sendMessage(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }

    if (RegExp(r"^(sendPhoto)$", caseSensitive: false).hasMatch(method)) {
      return await sendPhoto(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }

    if (RegExp(r"^(sendPoll)$", caseSensitive: false).hasMatch(method)) {
      return await sendPoll(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }

    if (RegExp(r"^(sendSticker)$", caseSensitive: false).hasMatch(method)) {
      return await sendSticker(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }

    if (RegExp(r"^(sendVenue)$", caseSensitive: false).hasMatch(method)) {
      return await sendVenue(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }

    if (RegExp(r"^(sendVideoNote)$", caseSensitive: false).hasMatch(method)) {
      return await sendVideoNote(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }

    if (RegExp(r"^(sendVideo)$", caseSensitive: false).hasMatch(method)) {
      return await sendVideo(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }
    if (RegExp(r"^(sendVoice)$", caseSensitive: false).hasMatch(method)) {
      return await sendVoice(
        parameters: parameters,
        callApiInvoke: callApiInvoke,
        telegramClientData: telegramClientData,
      );
    }

    telegram_client_scheme.Error error = telegram_client_scheme.Error.create(
      special_extra: "error",
      code: 500,
      message: "telegram_client_type_not_found",
      description: "",
    );
    if (isInvokeThrowOnError == true) {
      throw error;
    }
    return error;
  }
}
