// ignore_for_file: non_constant_identifier_names, camel_case_extensions

import 'dart:async';

import 'package:telegram_client/telegram_client/scheme/json_dart.dart'
    as scheme;
import 'package:telegram_client/telegram_client/telegram_client.dart';
import 'package:telegram_client/util/util.dart';

/// method MessageData
extension MessageDataDataOn on TelegramClient {
  Future<Map> message_Message({
    required Map message,
    required TelegramClientData telegramClientData,
    bool is_skip_reply_message = false,
  }) async {
    Map message_thread_json = {};
    if (message["message_thread_id"] is int &&
        message["message_thread_id"] > 0) {
      message_thread_json["message_thread_id"] = message["message_thread_id"];
    }
    Map message_sender_chat_json = {};

    if (message["sender_id"] is Map) {
      if (message["sender_id"]["@type"] == "messageSenderUser") {
        if (message["sender_id"]["user_id"] is int) {
          message_sender_chat_json["id"] = message["sender_id"]["user_id"];
          var res = await request(
            parameters: {
              "@type": "getUser",
              "user_id": message_sender_chat_json["id"],
            },
            telegramClientData: telegramClientData,
          );

          message_sender_chat_json.addAll(res.rawData);
        }
      }
      if (message["sender_id"]["@type"] == "messageSenderChat") {
        if (message["sender_id"]["chat_id"] is int) {
          message_sender_chat_json["id"] = message["sender_id"]["chat_id"];
          var res = await request(
            parameters: {
              "@type": "getChat",
              "chat_id": message_sender_chat_json["id"],
            },
            telegramClientData: telegramClientData,
          );

          message_sender_chat_json.addAll(res.rawData);
        }
      }
    }
    Map message_chat_json = {};

    if (message["chat_id"] is int) {
      message_chat_json["id"] = message["chat_id"];
      var res = await request(
        parameters: {
          "@type": "getChat",
          "chat_id": message_chat_json["id"],
        },
        telegramClientData: telegramClientData,
      );
      message_chat_json.addAll(res.rawData);
    }
    if (telegramClientData.is_bot == false) {
      message_sender_chat_json
          .removeWhere((key, value) => key == "phone_number");
      message_chat_json.removeWhere((key, value) => key == "phone_number");
    }
    Map msg = {
      "@type": "message",
      "id": message["id"],
      "message_id": TgUtils().messageTdlibToApi(message["id"]),
      ...message_thread_json,
      "from": message_sender_chat_json,
      "chat": message_chat_json,
      "date": message["date"],
      "is_outgoing": (message["is_outgoing"] == true),
      "content_type": "",
    };

    if (message["is_channel_post"] == true) {
      msg["chat"]["type"] = "channel";
    }

    if (message["reply_to"] is Map) {
      if (is_skip_reply_message == false) {
        if (message["reply_to"]["@type"] == "messageReplyToMessage") {
          int message_reply_to_chat_id = message["reply_to"]["chat_id"];
          int message_reply_to_message_id = message["reply_to"]["message_id"];
          if (message_reply_to_message_id > 0) {
            scheme.JsonScheme message_reply_to = await invoke(
              parameters: {
                "@type": "getRepliedMessage",
                "chat_id": message_reply_to_chat_id,
                "message_id": message["id"],
              },
              telegramClientData: telegramClientData,
            );

            msg["reply_to_message"] = await message_Message(
              message: message_reply_to.rawData,
              telegramClientData: telegramClientData,
              is_skip_reply_message: true,
            );
          }
        }
      }
    }

    if (message["edit_date"] is int && message["edit_date"] > 0) {
      msg["edit_date"] = message["edit_date"];
    }
    if (message["author_signature"] is String &&
        (message["author_signature"] as String).isNotEmpty) {
      msg["author_signature"] = message["author_signature"];
    }
    if (message["content"] is Map) {
      Map message_content = message["content"];
      if (message_content["@type"] == "messageText") {
        if (message_content["text"] is Map) {
          msg["text"] = message_content["text"]["text"];
        }
      }

      if (message_content["@type"] == "messageAnimation") {
        /// http://0.0.0.0:8080/classtd_1_1td__api_1_1animation.html
        Map animation = {};
        if (message["animation"] is Map) {
          if (message["animation"]["animation"] is Map) {
            Map message_animation = message["animation"]["animation"];
            animation["id"] = message_animation["id"];
            if (message_animation["local"] is Map) {
              animation["file_path"] = message_animation["local"]["path"];
              animation["is_downloading_active"] =
                  message_animation["local"]["is_downloading_active"];
              animation["is_downloading_completed"] =
                  message_animation["local"]["is_downloading_completed"];
            }
            if (message_animation["remote"] is Map) {
              animation["file_id"] = message_animation["remote"]["id"];
              animation["file_unique_id"] =
                  message_animation["remote"]["unique_id"];
              animation["file_size"] = message_animation["remote"]["size"];
            }
          }
          animation["width"] = message["animation"]["width"];
          animation["height"] = message["animation"]["height"];
          animation["duration"] = message["animation"]["duration"];
          // Map msg_content_thumbnail = {};
          animation["file_name"] = message["animation"]["file_name"];
          animation["mime_type"] = message["animation"]["mime_type"];
        }
        msg["animation"] = animation;
      }

      if (message_content["@type"] == "messagePhoto") {
        // http://0.0.0.0:8080/classtd_1_1td__api_1_1animation.html
        List photos = [];
        if (message_content["photo"] is Map) {
          List photo_sizes = message_content["photo"]["sizes"];

          for (var i = 0; i < photo_sizes.length; i++) {
            dynamic photo_data = photo_sizes[i];
            if (photo_data is Map) {
              if (photo_data["photo"] is Map) {
                Map jsonDataPhoto = {};

                Map message_photo = photo_data["photo"];
                jsonDataPhoto["id"] = message_photo["id"];
                if (message_photo["local"] is Map) {
                  jsonDataPhoto["file_path"] = message_photo["local"]["path"];
                  jsonDataPhoto["is_downloading_active"] =
                      message_photo["local"]["is_downloading_active"];
                  jsonDataPhoto["is_downloading_completed"] =
                      message_photo["local"]["is_downloading_completed"];
                }
                if (message_photo["remote"] is Map) {
                  jsonDataPhoto["file_id"] = message_photo["remote"]["id"];
                  jsonDataPhoto["file_unique_id"] =
                      message_photo["remote"]["unique_id"];
                  jsonDataPhoto["file_size"] = message_photo["remote"]["size"];
                }
                photos.add(jsonDataPhoto);
              }
            }
          }
        }
        msg["photo"] = photos;
      }

      if (message_content["caption"] is Map) {
        msg["caption"] = message_content["caption"]["text"];
      }
    }

    if (message["reply_markup"] is Map) {
      Map? reply_markup = TgUtils.replyMarkupTdlibToTgApi(
        replyMarkup: message["reply_markup"],
      );
      if (reply_markup != null) {
        msg["reply_markup"] = reply_markup;
      }
    }
    message.forEach((key, value) {
      if (value is bool) {
        msg[key] = value;
      }
    });

    return msg;
  }

  /// method MessageData
  FutureOr<Map?> message_toJson({
    required Map update,
    required TelegramClientData telegramClientData,
  }) async {
    // http://0.0.0.0:8704/classtd_1_1td__api_1_1message.html
    if (update["@type"] == "updateNewMessage") {
      Map message = update["message"];
      if ((message["is_outgoing"] == true)) {
        if (telegramClientData.is_bot) {
          return null;
        }
      }
      Map msg = await message_Message(
        message: message,
        telegramClientData: telegramClientData,
      );

      if (msg["chat"]["type"] == "channel") {
        return {
          "@type": "updateChannelPost",
          "channel_post": msg,
        };
      } else {
        return {
          "@type": "updateMessage",
          "update_message": msg,
        };
      }
    }

    return null;
  }
}
